using System;
using System.Collections.Generic;
using System.Text;
using Syncfusion.XlsIO;
using System.Drawing;

namespace Fox.Fbc.Pace.AdSales.PL.AdSalesClient.Reporting.DealFlowchartReport.ExcelExport
{
    public class XlsIOReportingHelper : IDisposable
    {
        private Dictionary<string, Font> _fonts = new Dictionary<string, Font>();
        private Dictionary<string, StringFormat> _stringFormats = new Dictionary<string, StringFormat>();
        private Dictionary<string, float> _fontIndentSizes = new Dictionary<string, float>();
        private Graphics _graphics;
        private Bitmap _bitmap;
        private bool _isDisposed;

        private float CalcMeasureStringMaxColumnWidth(IRange cell)
        {
            float ret = 0;
            if (cell.IsMerged)
            {
                int lastColumnIndex = cell.MergeArea.LastColumn;
                for (int i = cell.MergeArea.Column; i <= lastColumnIndex; i++)
                {
                    ret += cell.Worksheet.GetColumnWidthInPixels(i);
                }
            }
            else
            {
                ret = cell.Worksheet.GetColumnWidthInPixels(cell.Column);
            }

            if (cell.CellStyle.IndentLevel > 0 && cell.CellStyle.WrapText
                && (cell.CellStyle.HorizontalAlignment == ExcelHAlign.HAlignRight ||
                    cell.CellStyle.HorizontalAlignment == ExcelHAlign.HAlignLeft ||
                    cell.CellStyle.HorizontalAlignment == ExcelHAlign.HAlignGeneral))
            {
                const int fudgeFactor = 3;
                ret -= (cell.IndentLevel * this.GetFontIndentSize(cell.CellStyle.Font)) + fudgeFactor;
                if (ret < 0)
                {
                    ret = 0;
                }
            }

            return ret;
        }

        private StringFormat GetStringFormat(IStyle style)
        {
            StringAlignment hAlign;
            switch (style.HorizontalAlignment)
            {
                case ExcelHAlign.HAlignCenter:
                    hAlign = StringAlignment.Center;
                    break;
                case ExcelHAlign.HAlignRight:
                    hAlign = StringAlignment.Far;
                    break;
                case ExcelHAlign.HAlignLeft:
                case ExcelHAlign.HAlignGeneral:
                    hAlign = StringAlignment.Near;
                    break;
                default:
                    throw new ArgumentOutOfRangeException();
            }

            StringAlignment vAlign;
            switch (style.VerticalAlignment)
            {
                case ExcelVAlign.VAlignCenter:
                    vAlign = StringAlignment.Center;
                    break;
                case ExcelVAlign.VAlignBottom:
                    vAlign = StringAlignment.Far;
                    break;
                case ExcelVAlign.VAlignTop:
                    vAlign = StringAlignment.Near;
                    break;
                default:
                    throw new ArgumentOutOfRangeException();
            }

            StringFormat ret;
            string key = hAlign.ToString() + vAlign.ToString();
            if (!this._stringFormats.ContainsKey(key))
            {
                ret = (StringFormat)StringFormat.GenericTypographic.Clone();
                ret.FormatFlags = StringFormatFlags.LineLimit |
                                  StringFormatFlags.FitBlackBox |
                                  StringFormatFlags.MeasureTrailingSpaces;
                ret.Alignment = hAlign;
                ret.LineAlignment = vAlign;

                this._stringFormats.Add(key, ret);
            }
            else
            {
                ret = this._stringFormats[key];
            }

            return ret;
        }

        private Font GetFont(IFont font)
        {
            Font ret;
            string key = GetFontKey(font);
            if (!this._fonts.ContainsKey(key))
            {
                ret = new Font(font.FontName, font.Size, GetFontStyle(font));
                this._fonts.Add(key, ret);
            }
            else
            {
                ret = this._fonts[key];
            }
            return ret;
        }

        private static FontStyle GetFontStyle(IFont font)
        {
            return (font.Bold ? FontStyle.Bold : 0) |
                   (font.Italic ? FontStyle.Italic : 0) |
                   (font.Underline != ExcelUnderline.None ? FontStyle.Underline : 0) |
                   (font.Strikethrough ? FontStyle.Strikeout : 0);
        }

        private static string GetFontKey(IFont font)
        {
            return "1:" + font.FontName + "2:" + font.Size + "3:" + GetFontStyle(font);
        }

        private float GetFontIndentSize(IFont font)
        {
            float ret;
            string key = GetFontKey(font);
            if (!this._fontIndentSizes.ContainsKey(key))
            {
                ret = this.Graphics.MeasureString(" ", GetFont(font)).Width;
                this._fontIndentSizes.Add(key, ret);
            }
            else
            {
                ret = this._fontIndentSizes[key];
            }
            return ret;
        }

        private Graphics Graphics
        {
            get
            {
                if (this._graphics == null)
                {
                    this._bitmap = new Bitmap(1, 1);
                    this._graphics = System.Drawing.Graphics.FromImage(this._bitmap);
                }

                return this._graphics;
            }
        }

        public double CalcRowHeight(int rowIndex, IWorksheet worksheet, double? minimumRowHeightReturned)
        {
            if (this._isDisposed)
            {
                throw new ObjectDisposedException(this.GetType().Name);
            }

            double ret = 0;

            int lastColumnIndex = worksheet.UsedRange.LastColumn;
            for (int columnIndex = 1; columnIndex <= lastColumnIndex; columnIndex++)
            {
                if (!worksheet.Contains(rowIndex, columnIndex))
                {
                    continue;
                }

                IRange cell = worksheet.Range[rowIndex, columnIndex];
                if (string.IsNullOrEmpty(cell.Text))
                {
                    continue;
                }

                if (cell.IsMerged)
                {
                    if (cell.Row != rowIndex
                        || cell.LastRow != rowIndex)
                    {
                        throw new ArgumentOutOfRangeException();
                    }

                    columnIndex = cell.MergeArea.LastColumn + 1;
                }

                double rowHeight;
                if (cell.WrapText && !cell.HasNumber)
                {
                    rowHeight = this.Graphics.MeasureString(cell.Text,
                                            this.GetFont(cell.CellStyle.Font),
                                            new SizeF(this.CalcMeasureStringMaxColumnWidth(cell), float.MaxValue),
                                            this.GetStringFormat(cell.CellStyle)).Height;
                }
                else
                {
                    rowHeight = this.Graphics.MeasureString(cell.Text, this.GetFont(cell.CellStyle.Font)).Height;
                }

                ret = Math.Max(ret, rowHeight);
            }

            return Math.Ceiling(Math.Max(ret, minimumRowHeightReturned ?? ret));
        }

        #region IDisposable Members

        public void Dispose()
        {
            this.Dispose(true);
        }

        private void Dispose(bool isDisposing)
        {
            if (this._isDisposed)
            {
                return;
            }

            if (this._graphics != null)
            {
                this._graphics.Dispose();
                this._bitmap.Dispose();
                this._graphics = null;
            }

            if (this._fonts != null)
            {
                foreach (Font font in this._fonts.Values)
                {
                    font.Dispose();
                }
                this._fonts = null;
            }

            if (this._stringFormats != null)
            {
                foreach (StringFormat sf in this._stringFormats.Values)
                {
                    sf.Dispose();
                }
                this._stringFormats = null;
            }

            if (isDisposing)
            {
                GC.SuppressFinalize(this);
            }

            this._isDisposed = true;
        }

        ~XlsIOReportingHelper()
        {
            this.Dispose(false);
        }

        #endregion
    }
}